import logging
import pandas as pd


from openfisca_survey_manager.temporary import temporary_store_decorator  # type: ignore


log = logging.getLogger(__name__)


@temporary_store_decorator(file_name = 'erfs_fpr')
def build_variables_foyers_fiscal(temporary_store = None, year = None):

    assert temporary_store is not None
    assert year is not None

    individus = temporary_store['individus_{}'.format(year)]
    menages = temporary_store['menages_{}'.format(year)]

    individus['idfoy'] = individus['idfam'].copy()
    individus['quifoy'] = individus['quifam'].copy()

    foyers_fiscaux = individus[['idfoy','ident',]].drop_duplicates()
    foyers_fiscaux = pd.merge(
        menages[[
            'ident',
            'rev_financier_prelev_lib_imputes',
            'rev_fonciers_bruts',
            'rev_valeurs_mobilieres_bruts',
            'wprm',
            ]],
        foyers_fiscaux,
        how = 'inner',
        on = 'ident'
        )
    # première version pour splitter les revenus du capital du ménage dans les foyers fiscaux
    # on attribue l'ensemble des revenus du capital du ménage au foyer avec la personne ayant les plus hauts revenus
    # procédure à améliorer
    idfoy = (individus
        .sort_values(
            [
                'ident',
                'salaire_de_base',
                'traitement_indiciaire_brut',
                'retraite_brute'
                ],
            ascending = False
            )
        .groupby('ident')
        .first()
        .idfoy
        )
    foyers_fiscaux['revenu_categoriel_foncier'] = foyers_fiscaux['rev_fonciers_bruts'] * foyers_fiscaux.idfoy.isin(idfoy)
    foyers_fiscaux['revenus_capitaux_prelevement_forfaitaire_unique_ir'] = foyers_fiscaux['rev_valeurs_mobilieres_bruts'] * foyers_fiscaux.idfoy.isin(idfoy)
    foyers_fiscaux['rev_financier_prelev_lib_imputes'] = foyers_fiscaux['rev_financier_prelev_lib_imputes'] * foyers_fiscaux.idfoy.isin(idfoy)

    temporary_store[f"foyers_fiscaux_{year}"] = foyers_fiscaux



import os

import pytest

from openfisca_survey_manager.calibration import Calibration  # type: ignore
from openfisca_france_data import openfisca_france_data_location




@pytest.fixture
def location() -> str:
    return openfisca_france_data_location


def test_calibration(survey_scenario, fake_input_data, location, year: int = 2009):
    # On ititialise le survey scenario
    survey_scenario = survey_scenario(year)

    # On charge les données
    input_data = fake_input_data(year)

    # On initialise le survey scenario
    survey_scenario.init_from_data(data = dict(input_data_frame = input_data))

    # On fait la calibration
    parameters = dict(
        method = "logit",
        invlo = 3,
        up = 3,
        )

    pre_cal_weight = survey_scenario.calculate_variable("wprm", period = year)
    target_entity_count = pre_cal_weight.sum() * 1.123
    survey_scenario.calibrate(target_entity_count = target_entity_count, entity = "menage", parameters = parameters, period = year)

    assert pre_cal_weight * 1.123 == survey_scenario.calculate_variable("wprm", period = year)



import pandas as pd
from yaml import load, SafeLoader
import os
import re

from openfisca_core.periods import *

from openfisca_france import FranceTaxBenefitSystem

from openfisca_france_data.felin.input_data_builder.create_variables_individuelles import create_taux_csg_remplacement
from openfisca_france_data.common import create_revenus_remplacement_bruts

margin = 1

tax_benefit_system = FranceTaxBenefitSystem()
scenario = tax_benefit_system.new_scenario()

## First part : upwards (start from *_taxable, inverse to *_gross)

# Data creation

cd = os.path.dirname(__file__)
path = os.path.join(cd, "inversion", "remplacement_2021.yaml")
year = re.match(".*([0-9]{4}).yaml", path).group(1)

with open(path) as yaml:
    individus = pd.DataFrame.from_dict(load(yaml, Loader=SafeLoader))

# Inverse incomes from net to gross : the tested functions

create_taux_csg_remplacement(individus, period(year), tax_benefit_system)
create_revenus_remplacement_bruts(individus, period(year), tax_benefit_system)

# Test against chomage_brut_test

fails_chomage = [i for i in individus.index if abs(individus.loc[i]["chomage_brut"]-individus.loc[i]["chomage_brut_test"])>=margin]
fails_retraite = [i for i in individus.index if abs(individus.loc[i]["retraite_brute"]-individus.loc[i]["retraite_brute_test"])>=margin]

message = "".join(
    ["For test {}, found {} for chomage_brut, tested against {}.\n".format(i,individus.loc[i]["chomage_brut"],individus.loc[i]["chomage_brut_test"]) for i in fails_chomage]+
    ["For test {}, found {} for retraite_brute, tested against {}.\n".format(i,individus.loc[i]["retraite_brute"],individus.loc[i]["retraite_brute_test"]) for i in fails_retraite]
    )

assert len(fails_chomage) + len(fails_retraite) ==0, "Some tests have failed.\n" + message

## Second part : downwards (start from gross obtained from inversion, goes back to taxable)

# Initialize the survey scenario with the gross (inverted)

# init_single_entity(scenario, init_data)
# simulation = scenario.new_simulation()

# # Computes *_taxable back from inverted *_gross

# simulation.calculate('chomage_imposable', '2021-01') == 1000
# simulation.calculate('csg_deductible_chomage', '2021-01') == 0
# simulation.calculate('csg_imposable_chomage', '2021-01') == 0
# simulation.calculate('crds_chomage', '2021-01') == 1

